@@ -150,7 +150,7 @@ impl GenerateCI {
             pyproject_toml,
             project_layout,
             ..
-        } = ProjectResolver::resolve(self.manifest_path.clone(), cargo_options)?;
+        } = ProjectResolver::resolve(self.manifest_path.clone(), cargo_options, false)?;
         let pyproject = pyproject_toml.as_ref();
         let bridge = find_bridge(&cargo_metadata, pyproject.and_then(|x| x.bindings()))?;
         let project_name = pyproject
@@ -169,14 +169,13 @@ impl GenerateCI {
         bridge_model: &BridgeModel,
         sdist: bool,
     ) -> Result<String> {
-        let is_abi3 = matches!(bridge_model, BridgeModel::BindingsAbi3 { .. });
+        let is_abi3 = bridge_model.is_abi3();
         let is_bin = bridge_model.is_bin();
         let setup_python = self.pytest
             || matches!(
                 bridge_model,
                 BridgeModel::Bin(Some(_))
-                    | BridgeModel::Bindings { .. }
-                    | BridgeModel::BindingsAbi3 { .. }
+                    | BridgeModel::PyO3 { .. }
                     | BridgeModel::Cffi
                     | BridgeModel::UniFfi
             );
@@ -373,7 +372,7 @@ jobs:\n",
         with:
           target: ${{{{ matrix.platform.target }}}}
           args: --release --out dist{maturin_args}
-          sccache: 'true'
+          sccache: ${{{{ !startsWith(github.ref, 'refs/tags/') }}}}
 "
             ));
             let maturin_action_args = match platform {
@@ -387,13 +386,21 @@ jobs:\n",
             }
             if is_abi3 {
                 // build free-threaded wheel for python3.13t
+                if matches!(platform, Platform::Windows) {
+                    conf.push_str(
+                        "      - uses: actions/setup-python@v5
+        with:
+          python-version: 3.13t\n",
+                    );
+                    conf.push_str("          architecture: ${{ matrix.platform.target }}\n");
+                }
                 conf.push_str(&format!(
                     "      - name: Build free-threaded wheels
         uses: PyO3/maturin-action@v1
         with:
           target: ${{{{ matrix.platform.target }}}}
           args: --release --out dist{maturin_args} -i python3.13t
-          sccache: 'true'
+          sccache: ${{{{ !startsWith(github.ref, 'refs/tags/') }}}}
 "
                 ));
                 if !maturin_action_args.is_empty() {
@@ -621,7 +628,7 @@ jobs:\n",
         if !self.skip_attestation {
             conf.push_str(
                 r#"      - name: Generate artifact attestation
-        uses: actions/attest-build-provenance@v1
+        uses: actions/attest-build-provenance@v2
         with:
           subject-path: 'wheels-*/*'
 "#,
@@ -665,7 +672,7 @@ jobs:\n",
 #[cfg(test)]
 mod tests {
     use super::GenerateCI;
-    use crate::{Bindings, BridgeModel};
+    use crate::{bridge::PyO3Crate, Abi3Version, BridgeModel, PyO3};
     use expect_test::expect;
     use semver::Version;
 
@@ -674,9 +681,11 @@ mod tests {
         let conf = GenerateCI::default()
             .generate_github(
                 "example",
-                &BridgeModel::Bindings(Bindings {
-                    name: "pyo3".to_string(),
+                &BridgeModel::PyO3(PyO3 {
+                    crate_name: PyO3Crate::PyO3,
                     version: Version::new(0, 23, 0),
+                    abi3: None,
+                    metadata: None,
                 }),
                 true,
             )
@@ -729,7 +738,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -760,7 +769,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -788,7 +797,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -814,7 +823,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -851,7 +860,7 @@ mod tests {
                 steps:
                   - uses: actions/download-artifact@v4
                   - name: Generate artifact attestation
-                    uses: actions/attest-build-provenance@v1
+                    uses: actions/attest-build-provenance@v2
                     with:
                       subject-path: 'wheels-*/*'
                   - name: Publish to PyPI
@@ -870,14 +879,12 @@ mod tests {
         let conf = GenerateCI::default()
             .generate_github(
                 "example",
-                &BridgeModel::BindingsAbi3 {
-                    bindings: Bindings {
-                        name: "pyo3".to_string(),
-                        version: Version::new(0, 23, 0),
-                    },
-                    major: 3,
-                    minor: 7,
-                },
+                &BridgeModel::PyO3(PyO3 {
+                    crate_name: PyO3Crate::PyO3,
+                    version: Version::new(0, 23, 0),
+                    abi3: Some(Abi3Version::Version(3, 7)),
+                    metadata: None,
+                }),
                 false,
             )
             .unwrap()
@@ -929,14 +936,14 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -967,14 +974,14 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -1002,13 +1009,17 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
+                  - uses: actions/setup-python@v5
+                    with:
+                      python-version: 3.13t
+                      architecture: ${{ matrix.platform.target }}
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1034,13 +1045,13 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1062,7 +1073,7 @@ mod tests {
                 steps:
                   - uses: actions/download-artifact@v4
                   - name: Generate artifact attestation
-                    uses: actions/attest-build-provenance@v1
+                    uses: actions/attest-build-provenance@v2
                     with:
                       subject-path: 'wheels-*/*'
                   - name: Publish to PyPI
@@ -1084,14 +1095,12 @@ mod tests {
         }
         .generate_github(
             "example",
-            &BridgeModel::BindingsAbi3 {
-                bindings: Bindings {
-                    name: "pyo3".to_string(),
-                    version: Version::new(0, 23, 0),
-                },
-                major: 3,
-                minor: 7,
-            },
+            &BridgeModel::PyO3(PyO3 {
+                crate_name: PyO3Crate::PyO3,
+                version: Version::new(0, 23, 0),
+                abi3: Some(Abi3Version::Version(3, 7)),
+                metadata: None,
+            }),
             false,
         )
         .unwrap()
@@ -1143,14 +1152,14 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -1181,14 +1190,14 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -1216,13 +1225,17 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
+                  - uses: actions/setup-python@v5
+                    with:
+                      python-version: 3.13t
+                      architecture: ${{ matrix.platform.target }}
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1248,13 +1261,13 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Build free-threaded wheels
                     uses: PyO3/maturin-action@v1
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist -i python3.13t
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1294,9 +1307,11 @@ mod tests {
         let conf = gen
             .generate_github(
                 "example",
-                &BridgeModel::Bindings(Bindings {
-                    name: "pyo3".to_string(),
+                &BridgeModel::PyO3(PyO3 {
+                    crate_name: PyO3Crate::PyO3,
                     version: Version::new(0, 23, 0),
+                    abi3: None,
+                    metadata: None,
                 }),
                 true,
             )
@@ -1349,7 +1364,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter --zig
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -1405,7 +1420,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -1463,7 +1478,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1499,7 +1514,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist --find-interpreter
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1544,7 +1559,7 @@ mod tests {
                 steps:
                   - uses: actions/download-artifact@v4
                   - name: Generate artifact attestation
-                    uses: actions/attest-build-provenance@v1
+                    uses: actions/attest-build-provenance@v2
                     with:
                       subject-path: 'wheels-*/*'
                   - name: Publish to PyPI
@@ -1608,7 +1623,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -1636,7 +1651,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                       manylinux: musllinux_1_2
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
@@ -1660,7 +1675,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1683,7 +1698,7 @@ mod tests {
                     with:
                       target: ${{ matrix.platform.target }}
                       args: --release --out dist
-                      sccache: 'true'
+                      sccache: ${{ !startsWith(github.ref, 'refs/tags/') }}
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1720,7 +1735,7 @@ mod tests {
                 steps:
                   - uses: actions/download-artifact@v4
                   - name: Generate artifact attestation
-                    uses: actions/attest-build-provenance@v1
+                    uses: actions/attest-build-provenance@v2
                     with:
                       subject-path: 'wheels-*/*'
                   - name: Publish to PyPI
